# -----------------------------------------------------------------------------
# Routines that related to spectrum axes.
#
import string

import sputil
import tkutil

# -----------------------------------------------------------------------------
#
def unique_nucleus_axis(spectrum, nucleus):

  axis = None
  for a in range(spectrum.dimension):
    if spectrum.nuclei[a] == nucleus:
      if axis == None:
        axis = a
      else:
        return None
  return axis

# -----------------------------------------------------------------------------
#
def second_nucleus_axis(spectrum, nucleus, not_this_axis):

  for a in range(spectrum.dimension):
    if a != not_this_axis and spectrum.nuclei[a] == nucleus:
      return a
  return None

# ----------------------------------------------------------------------------
#
def proton_axes(spectrum):

  axes = []
  nuclei = spectrum.nuclei
  for a in range(spectrum.dimension):
    if nuclei[a] == '1H':
      axes.append(a)
  return axes

# ----------------------------------------------------------------------------
#
def heavy_atom_axes(spectrum):

  axes = []
  nuclei = spectrum.nuclei
  for a in range(spectrum.dimension):
    if nuclei[a] != '1H':
      axes.append(a)
  return axes

# ----------------------------------------------------------------------------
#
def chn_axes(spectrum):

  if hasattr(spectrum, 'chn_axes'):
    return spectrum.chn_axes
  
  if spectrum.dimension < 3:
    return None
  
  aC = unique_nucleus_axis(spectrum, '13C')
  aH = unique_nucleus_axis(spectrum, '1H')
  aN = unique_nucleus_axis(spectrum, '15N')
  if aC == None or aH == None or aN == None:
    return None

  spectrum.chn_axes = (aC, aH, aN)
  
  return aC, aH, aN

# ----------------------------------------------------------------------------
# Return axes for hnca or hnha type experiments (aH, aN, aX)
#
def amide_axes(spectrum):

  if hasattr(spectrum, 'hnx_axes'):
    return spectrum.hnx_axes
  
  if spectrum.dimension < 3:
    return None
  
  aN = unique_nucleus_axis(spectrum, '15N')
  if aN == None:
    return None
  
  aH = unique_nucleus_axis(spectrum, '1H')
  if aH == None:
    aH = attached_proton_axis(spectrum, aN)
    if aH == None:
      return None
    
  aX = 3 - aH - aN

  spectrum.hnx_axes = (aH, aN, aX)
  
  return spectrum.hnx_axes

# ----------------------------------------------------------------------------
#
def hn_axes(spectrum):

  if hasattr(spectrum, 'hn_axes'):
    return spectrum.hn_axes
  
  aH = unique_nucleus_axis(spectrum, '1H')
  aN = unique_nucleus_axis(spectrum, '15N')
  if aH == None or aN == None:
    return None

  spectrum.hn_axes = (aH, aN)
  
  return aH, aN

# ----------------------------------------------------------------------------
# An option menu of spectrum axes
#
class axis_menu(tkutil.option_menu):

  def __init__(self, parent, spectrum, nucleus = None):

    axes = []
    for a in range(spectrum.dimension):
      axis_nucleus = spectrum.nuclei[a]
      if nucleus == None or axis_nucleus == nucleus: 
        axes.append('w%d %s' % (a+1, axis_nucleus))

    tkutil.option_menu.__init__(self, parent, '', axes)

  # -------------------------------------------------------------------------
  # Parse wN of axis label to return axis number
  #
  def chosen_axis(self):

    axis_text = self.get()
    if len(axis_text) < 2:
      return None
    return string.atoi(axis_text[1]) - 1

# ----------------------------------------------------------------------------
# An option menu of all possible spectrum axis orders
#
class axis_order_menu(tkutil.option_menu):

  def __init__(self, parent, spectrum, initial_order):

    self.nuclei = spectrum.nuclei
    orders = []
    for order in self.permutations(range(spectrum.dimension)):
      orders.append(self.order_text(order))

    if initial_order:
      initial = self.order_text(initial_order)
    else:
      initial = None
      
    tkutil.option_menu.__init__(self, parent, '', orders, initial)

  # -------------------------------------------------------------------------
  #
  def order_text(self, order):

    text = ''
    for axis in order:
      text = text + 'w%d %s  ' % (axis+1, self.nuclei[axis])
    return text

  # -------------------------------------------------------------------------
  #
  def axis_order(self):

    order_text = self.get()
    for order in self.permutations(range(len(self.nuclei))):
      if order_text == self.order_text(order):
        return order

  # -------------------------------------------------------------------------
  #
  def permutations(self, set):

    if len(set) <= 1:
      return (tuple(set),)

    perms = []
    plist = self.permutations(set[1:])
    e = set[0]
    for i in range(len(set)):
      for p in plist:
        perms.append(p[:i] + (e,) + p[i:])
    return tuple(perms)

# -----------------------------------------------------------------------------
#
def has_attached_proton_axis(spectrum, heavy_axis):

  key = 'w%d_attached_proton_axis' % (heavy_axis + 1)
  a = spectrum.saved_value(key)
  if a:
    a = string.atoi(a)
  return a

# -----------------------------------------------------------------------------
#
def set_attached_proton_axis(spectrum, heavy_axis, proton_axis):

  key = 'w%d_attached_proton_axis' % (heavy_axis + 1)
  spectrum.save_value(key, repr(proton_axis))

# -----------------------------------------------------------------------------
#
def attached_proton_axis(spectrum, heavy_axis):

  a = has_attached_proton_axis(spectrum, heavy_axis)
  if a == None:
    d = tkutil.the_dialog(attached_proton_dialog)
    d.get_attached_axis(spectrum, heavy_axis)
    
  return has_attached_proton_axis(spectrum, heavy_axis)

# -----------------------------------------------------------------------------
#
class attached_proton_dialog(tkutil.Dialog):

  def __init__(self):

    tkutil.Dialog.__init__(self, 'Attached Proton')

    am = tkutil.option_menu(self.top, 'Attached proton axis: ', ())
    am.frame.pack(side = 'top')
    self.axis_menu = am

    br = tkutil.button_row(self.top,
			   ('Ok', self.ok_cb),
                           ('Cancel', self.close_cb),
                           ('Help', sputil.help_cb('LabelledAxis')),
			   )
    br.frame.pack(side = 'top', anchor = 'w')

  # ---------------------------------------------------------------------------
  #
  def ok_cb(self):

    axis_name = self.axis_menu.get()
    axis_number = {'w1':0, 'w2':1, 'w3':2, 'w4':3}
    if axis_number.has_key(axis_name):
      attached_axis = axis_number[axis_name]
      set_attached_proton_axis(self.spectrum, self.heavy_axis, attached_axis)
    self.close_cb()

  # ---------------------------------------------------------------------------
  #
  def set_spectrum_heavy_axis(self, spectrum, heavy_axis):

    self.spectrum = spectrum
    self.heavy_axis = heavy_axis
    
    request = ('Attached proton axis for w%d %s of %s: '
               % (heavy_axis+1, spectrum.nuclei[heavy_axis], spectrum.name))
    self.axis_menu.label['text'] = request

    self.axis_menu.remove_all_entries()
    for a in range(spectrum.dimension):
      if spectrum.nuclei[a] == '1H':
        axis_name = 'w%d' % (a+1)
        self.axis_menu.add_entry(axis_name)
        self.axis_menu.set(axis_name)

  # ---------------------------------------------------------------------------
  #
  def get_attached_axis(self, spectrum, heavy_axis):

    self.set_spectrum_heavy_axis(spectrum, heavy_axis)
    self.show_window(1)
    self.return_when_closed()

# -----------------------------------------------------------------------------
#
def show_attached_axis_dialog():

  spectrum = sputil.selected_spectrum()
  if spectrum:
    axes = heavy_atom_axes(spectrum)
    if len(axes) == 1:
      d = tkutil.the_dialog(attached_proton_dialog)
      d.set_spectrum_heavy_axis(spectrum, axes[0])
      d.show_window(1)
